# Install for Development

<!-- TOC -->

1. [Install for Development](#install-for-development)
   1. [Prerequisites](#prerequisites)
   2. [Preparation](#preparation)
      1. [Setting up the database](#setting-up-the-database)
   3. [Configuration](#configuration)
   4. [Running Engine](#running-engine)
   5. [Daemonized Engine](#daemonized-engine)
      1. [Running with Systemd](#running-with-systemd)
      2. [Running with Supervisor](#running-with-supervisor)
   6. [Logging](#logging)
   7. [Read more](#read-more)

<!-- /TOC -->

## Prerequisites

Aura Engine runs on any modern Debian-based OS. It requires at least

- [Python 3.8+](https://www.python.org/downloads/release/python-380/)
- [`pip`](https://pip.pypa.io/en/stable/)
- [`git`](https://git-scm.com/)
- 'python3.8-venv' or [`virtualenv`](https://pypi.org/project/virtualenv/)
- [PostgreSQL 12+](https://www.postgresql.org/)


**Setting up the project structure**

Then clone all `engine` related projects to some development folder. In production we recommend `/opt/aura/` as the system base.

```bash
mkdir ~/code/aura
cd ~/code/aura
git clone https://gitlab.servus.at/autoradio/engine
git clone https://gitlab.servus.at/autoradio/engine-core
git clone https://gitlab.servus.at/autoradio/engine-api
     ...
```

If you are developing engine you also might want the projects `steering`, `dashboard` and `tank` in there, or you'll start them with Docker Compose (see [how to run Aura Web with Docker](https://gitlab.servus.at/autoradio/meta) in the [meta](https://gitlab.servus.at/autoradio/meta) repository)

## Preparation

In case of an development environment, create a virtual environment for your Python dependencies:

```shell
python3.8 -m venv python
```

No worries, the resulting `python` directory is excluded from the git respostory.

To activate that environment, run

```shell
source python/bin/activate
```

Then, install the required dependencies

```shell
pip install -r requirements.txt
```

Create your base configuration from the sample configuration

```shell
# Development
cp config/sample-development.engine.ini config/engine.ini
# Production
cp config/sample-production.engine.ini config/engine.ini
```

### Setting up the database

Engine requires a PorstgreSQL database to cache any programme info locally:

```bash
# Additional Python packages for PostgreSQL
pip3 install -r contrib/postgresql-requirements.txt
# Create database and user (change password in script)
sudo -u postgres psql -f contrib/postgresql-create-database.sql
```

> Important: You might want to change the password for the database user created by the relevant script.

## Configuration

In your development environment edit following file to configure the engine:

```shell
./config/engine.ini
```

> Please note, if some configuration exists under `/etc/aura/engine.ini` the configuration by default is drawn from there. This overrides any configuration located in `./engine/config`.

While the configuration file has plenty of options, you only need to set a few mandatory ones, given you are running the other components (such as 'engine-core', "engine-api" etc.) at their default settings too.

Required modifications are:
    - The password `db_pass` for the local database holding scheduling information
    - The app secret `api_tank_secret` for connecting to [AURA Tank](https://gitlab.servus.at/aura/tank)

## Running Engine

If you have defined a virtual env during the installation step you'll need to activate it first. You'll have to do so whenever you gonna start your development environment:

```shell
source python/bin/activate
```

There's a convencience script `run.sh` to get Engine components started:

```shell
engine$ ./run.sh
```

Keep in mind you'll also need to start Engine Core separately.

> Note it should not matter in which order you start Engine and Engine Core.

```shell
engine-core$ ./run.sh
```

Last but not least, Engine API is the target service to store playlogs, health information and details for the [Studio Clock](https://gitlab.servus.at/aura/dashboard-clock).

```shell
engine-core$ ./run.sh
```

In order to have the complete Engine experience, other AURA Components are required to be running too. Checkout the [Meta Repository](https://gitlab.servus.at/aura/meta) on how to run for example AURA Web using Docker Compose.

## Daemonized Engine

For this you can utilize either [Systemd](https://systemd.io/) or [Supervisor](http://supervisord.org/). Please check the their manuals on how to use these services.

The daemon configs are expecting you run engine under the user `engineuser` and being located under `/opt/aura/engine`, `/opt/aura/engine-api` and `/opt/aura/engine-core` respectively. Do prepare the project root and permissions you can use the script `scripts/initialize-systemd.sh`. To create the matching user and audio group membership run `scripts/create-engineuser.sh`.

### Running with Systemd

Copy the unit files in  `/opt/aura/engine/config/systemd/aura-engine.service` to your systemd unit directory, and reload the systemd daemon:

```shell
cp /opt/aura/engine/config/systemd/* /etc/systemd/system/
systemctl daemon-reload
```

### Running with Supervisor

Now, given you are in the engine's home directory like `/opt/aura/engine/`, simply type following to start the services:

```shell
supervisord
```

This picks up the supervisor configuration provided in the local `supervisord.conf` and the service configurations located in `config/supervisor/*.conf`.

Then you'll need to reload the supervisor configuration using `sudo`:

```shell
sudo supervisorctl reload
```

## Logging

All Engine logs can be found under `./logs`.

## Read more

- [Bare Metal Installation](docs/bare-metal-installation.md)
- [Developer Guide](docs/developer-guide.md)
- [Setting up the Audio Store [meta]](https://gitlab.servus.at/aura/meta/-/blob/master/docs/administration/setup-audio-store.md)
